@@ -12,14 +12,14 @@
 #define	__column_h
 
 #if HAVE_CONFIG_H
-#include <conf.h>
+#include "conf.h"
 #endif
 
 #ifdef HAVE_STRING_H
 #include <string.h>
 #endif
 
-#include <prsetup.h>
+#include "prsetup.h"
 _PRbeg
 
 void ColumnLoad _PRx((char *ibuf, int size, int n, int convert, void *obuf));
