@@ -10,7 +10,7 @@
 
 #include <filter.h>
 
-#if HAVE_CYGWIN
+#if HAVE_CYGWIN||HAVE_MINGW32
 #include <windows.h>
 #endif
 
@@ -271,7 +271,11 @@ Filter FilterOpen(header, string, mode)
   filter->mode=xstrdup(mode);
 
   /* assume we will use an index */
+#if HAVE_MINGW32==0
   filter->doidx = 1;
+#else
+  filter->doidx = 0;
+#endif
 
   /* process the mode string */
   if( (s = xstrdup(filter->mode)) ){
@@ -283,12 +287,14 @@ Filter FilterOpen(header, string, mode)
       filter->evsect = xstrdup(tbuf);
     if( keyword(s, "lexonly", tbuf, SZ_LINE) )
       lexonly = 1;
+#if HAVE_MINGW32==0
     if( keyword(s, "idx", tbuf, SZ_LINE) ){
       if( istrue(tbuf) )
 	filter->doidx = 1;
       else if( isfalse(tbuf) )
 	filter->doidx = -1;
     }
+#endif
   }
   if( s ) xfree(s);
 
@@ -326,24 +332,31 @@ Filter FilterOpen(header, string, mode)
   }
 
   /* determine which type of process execution we do */
-  filter->ptype = DEFAULT_FILTER_PTYPE;
-  *tbuf = '\0';
-  if( (s=(char *)getenv("FILTER_PTYPE")) ){
-    strcpy(tbuf, s);
-  }
-  if( !*tbuf && (s=xstrdup(filter->mode)) ){
-    keyword(s, "ptype", tbuf, SZ_LINE);
-    if( s ) xfree(s);
-  }
-  if( *tbuf ){
-    if( *tbuf == 'p' )
-      filter->ptype = PTYPE_PROCESS;
-    else if( *tbuf == 'c' )
-      filter->ptype = PTYPE_CONTAINED;
+  switch(filter->method){
+  case METHOD_C:
+    filter->ptype = DEFAULT_FILTER_PTYPE;
+    *tbuf = '\0';
+    if( (s=(char *)getenv("FILTER_PTYPE")) ){
+      strcpy(tbuf, s);
+    }
+    if( !*tbuf && (s=xstrdup(filter->mode)) ){
+      keyword(s, "ptype", tbuf, SZ_LINE);
+      if( s ) xfree(s);
+    }
+    if( *tbuf ){
+      if( *tbuf == 'p' )
+	filter->ptype = PTYPE_PROCESS;
+      else if( *tbuf == 'c' )
+	filter->ptype = PTYPE_CONTAINED;
 #ifdef USE_DL
-    else if( *tbuf == 'd' )
-      filter->ptype = PTYPE_DYNAMIC;
+      else if( *tbuf == 'd' )
+	filter->ptype = PTYPE_DYNAMIC;
 #endif
+    }
+    break;
+  default:
+    filter->ptype = PTYPE_PROCESS;
+    break;
   }
 
   /* determine region paint mode */
@@ -416,6 +429,7 @@ Filter FilterOpen(header, string, mode)
     goto done;
 
   /* for event filtering, we can use indexes */
+#if HAVE_MINGW32==0
   if( filter->type == TYPE_EVENTS ){
     /* run index grammar and see if we can get help from indexes */
     if( filter->doidx == 1 ){
@@ -441,6 +455,7 @@ Filter FilterOpen(header, string, mode)
       goto done;
     }
   }
+#endif
 
   /* lexonly means we only wanted to run the lexer */
   if( lexonly )
@@ -470,15 +485,18 @@ Filter FilterOpen(header, string, mode)
   if( !FilterProgCompile(filter) )
     goto error;
 
-  if( filter->debug < 2 ){
+  if( filter->debug >= 2 ) goto done;
+
+  switch(filter->method){
+  case METHOD_C:
     switch(filter->ptype){
     case PTYPE_PROCESS:
     case PTYPE_CONTAINED:
       switch(filter->pipeos){
       case PIPE_WIN32:
-#if HAVE_CYGWIN
+#if HAVE_CYGWIN||HAVE_MINGW32
 	if( !WinProcessOpen(filter->prog, 
-	    &(filter->ihandle), &(filter->ohandle), &(filter->process)) )
+	     &(filter->ihandle), &(filter->ohandle), &(filter->process)) )
 	  goto error;
 #else
 	gerror(stderr, "internal error: no WinProcess without Windows");
@@ -487,20 +505,23 @@ Filter FilterOpen(header, string, mode)
 	break;
       default:
 	if( !ProcessOpen(filter->prog, 
-		    &(filter->ichan), &(filter->ochan), &(filter->pid)) )
+	     &(filter->ichan), &(filter->ochan), &(filter->pid)) )
 	  goto error;
 	break;
       }
       break;
 #ifdef USE_DL
     case PTYPE_DYNAMIC:
-     if( !(filter->dl=DLOpen(filter->prog)) )
-       goto error;
-     break;
+      if( !(filter->dl=DLOpen(filter->prog)) )
+	goto error;
+      break;
 #endif
     default:
-      goto error;
+	goto error;
     }
+    break;
+  default:
+    goto error;
   }
 
   /* return the good news */
@@ -533,7 +554,7 @@ int FilterEvents(filter, ebuf, esize, n, vbuf)
 {
   int i;
   int flag;
-  int get, got;
+  int get=0, got=0;
   int bytes;
   char *eptr;
   char *etop;
@@ -600,7 +621,7 @@ int FilterEvents(filter, ebuf, esize, n, vbuf)
   case PTYPE_CONTAINED:
     switch(filter->pipeos){
     case PIPE_WIN32:
-#if HAVE_CYGWIN
+#if HAVE_CYGWIN||HAVE_MINGW32
       /* write events to the filter process */
       bytes = n*filter->evsize;
       if((got=WinProcessWrite(filter->ohandle, obuf, bytes)) != bytes){
@@ -739,7 +760,7 @@ int FilterImage(filter, x0, x1, y0, y1, block, masks, nreg)
   case PTYPE_CONTAINED:
     switch(filter->pipeos){
     case PIPE_WIN32:
-#if HAVE_CYGWIN
+#if HAVE_CYGWIN||HAVE_MINGW32
       /* write command to process */
       snprintf(tbuf, SZ_LINE-1, "%d %d %d %d %d\n", x0, x1, y0, y1, block);
       WinProcessWrite(filter->ohandle, tbuf, (int)strlen(tbuf));
@@ -809,7 +830,7 @@ int FilterClose(filter)
     /* close the filter process */
     switch(filter->pipeos){
     case PIPE_WIN32:
-#if HAVE_CYGWIN
+#if HAVE_CYGWIN||HAVE_MINGW32
       if( filter->process ) WinProcessClose(filter->process, &status);
 #else
       gerror(stderr, "internal error: no WinProcess without Windows");
@@ -880,7 +901,9 @@ int FilterClose(filter)
     if( filter->g )         xfree(filter->g);
   }
   /* free index information */
+#if HAVE_MINGW32==0
   idxendparser();
+#endif
   /* free main struct */
   xfree(filter);
   /* return the status from closing the sub-process */
