@@ -37,7 +37,8 @@ static void *_FunTableBin(fun, buf, plist)
   int dofilt;			/* true if we can filter */
   int dovcol;			/* true if we use a value column */
   int voffset;			/* offset into event record for vcol */
-  int vsize;			/* size of vcol in bytes */
+  /* not used, so avoid gcc compiler warning */
+  /* int vsize; */		/* size of vcol in bytes */
   int vtype;			/* data type of event vcol */
   int imdim;			/* total dims of image buffer */
   int itsnan;			/* if value is a nan */
@@ -111,9 +112,10 @@ static void *_FunTableBin(fun, buf, plist)
   /* optional value column for binning */
   if( fun->vbin >= 0 ){
     dovcol = 1;
-    voffset = fun->header->table->col[fun->bin[2]].offset;
-    vsize = fun->header->table->col[fun->bin[2]].size;
-    vtype = fun->header->table->col[fun->bin[2]].type;
+    voffset = fun->header->table->col[fun->vbin].offset;
+    /* not used, so avoid gcc compiler warning */
+    /* vsize = fun->header->table->col[fun->vbin].size; */
+    vtype = fun->header->table->col[fun->vbin].type;
   }
   else{
     dovcol = 0;
