@@ -237,7 +237,7 @@ static void _FunBlank(fun, type, buf)
   
 #ifdef ANSI_FUNC
 static void *
-_FunTextRead(Fun fun, char *buf, size_t size, size_t get,  size_t *got)
+_FunTextRead(Fun fun, char *buf, size_t UNUSED(size), size_t get,  size_t *got)
 #else
 static void *_FunTextRead(fun, buf, size, get,  got)
      Fun fun;
@@ -258,9 +258,6 @@ static void *_FunTextRead(fun, buf, size, get,  got)
   Parse parser=NULL;
   ParsedLine line;
 
-  /* avoid -W unused parameter warning */
-  if( 0 ) size = size;
-
   /* sanity checks
      NB: the order of ascii columns should be identical to the fitsy columns
      (since the latter was generated by the former) */
@@ -1384,9 +1381,9 @@ int FunTableRowPut(fun, rows, nrow, idx, plist)
 */
 #ifdef ANSI_FUNC
 off_t
-FunTableRowSeek(Fun fun, int nrow, char *plist)
+FunTableRowSeek(Fun fun, int nrow, char *UNUSED(plist))
 #else
-off_t FunTableRowPut(fun, nrow, plist)
+off_t FunTableRowSeek(fun, nrow, plist)
      Fun fun;
      int nrow;
      char *plist;
@@ -1394,9 +1391,6 @@ off_t FunTableRowPut(fun, nrow, plist)
 {
   off_t ipos, opos;
 
-  /* avoid -W unused parameter warning */
-  if( 0 ) plist = plist;
-
   /* gotta have it */
   if( !_FunValid(fun) )
     return (off_t)-1;
