@@ -234,7 +234,7 @@ HCProcess(hc, s)
   }
 
   /* add to expression */
-  if( hc->tbuf ){
+  if( *hc->tbuf ){
     len = strlen(tbuf);
     /* make sure we have ennough room */
     if( (len + hc->clen + 1) >= hc->maxlen ){
@@ -485,7 +485,7 @@ int FunTextParam(pdelims, lbuf, kbuf, vbuf, cbuf, maxlen)
       break;
     /* look for comment */
     case 6:
-      if( (*tptr == '/') ){
+      if( *tptr == '/' ){
 	tptr++;
 	hstate = 7;
 	len = 0;
