@@ -256,6 +256,7 @@ static int gmemwrite(gio, buf, size, n)
   return(n/size);
 }
 
+#if HAVE_MINGW32==0
 /* 
  *
  * gexec -- open a connected subprocess with bidirectional
@@ -377,6 +378,7 @@ static GIO gopenpipe(name, mode)
   gio->extend = 0;
   return(gio);
 }
+#endif
 
 /*
  *
@@ -1077,6 +1079,11 @@ static GIO gopenfile(name, mode)
   char *s;
   GIO gio;
 
+#if HAVE_MINGW32
+  /* binary mode for mingw */
+  _fmode = _O_BINARY;
+#endif
+
   /* allocate a new struct */
   if( (gio = gnew()) == NULL )
     return(NULL);
@@ -1162,9 +1169,22 @@ GIO gopen(fname, mode)
   /* expand environment variables to get working filename */
   ExpandEnv (tname, name, SZ_LINE-1);
 
+  /* look for an explicit file */
+  if( !strncasecmp(name, "file:", 5)   ||
+      !strncasecmp(name, "unfile:", 7) || strchr(mode, 'u') )
+    gio = gopenfile(name, mode);
+  /* look for stdio */
+  else if( (!strncasecmp(name, "stdin", 5)   || 
+	    !strncasecmp(name, "stdout", 6)) &&
+  	    !(gext(name) && !strcmp(gext(name),".gz")) )
+    gio = gopenstdio(name, mode);
+  else if( (*name == '-') && ((*(name+1) == '\0') || (*(name+1) == '[')) )
+    gio = gopenstdio(name, mode);
+#if HAVE_MINGW32==0
   /* look for command pipe */
-  if( !strncasecmp(name, "pipe:", 5) || strchr(mode, 'p') )
+  else if( !strncasecmp(name, "pipe:", 5) || strchr(mode, 'p') )
     gio = gopenpipe(name, mode);
+#endif
 #ifdef HAVE_SYS_MMAN_H
   /* look for a mapped file */
   else if( !strncasecmp(name, "mmap:", 5) || strchr(mode, 'm') )
@@ -1191,17 +1211,6 @@ GIO gopen(fname, mode)
       gio = gmemopen(buf, len, mode);
     }
   }
-  /* look for an explicit file */
-  else if( !strncasecmp(name, "file:", 5)   ||
-           !strncasecmp(name, "unfile:", 7) || strchr(mode, 'u') )
-    gio = gopenfile(name, mode);
-  /* look for stdio */
-  else if( (!strncasecmp(name, "stdin", 5)   || 
-	    !strncasecmp(name, "stdout", 6)) &&
-  	    !(gext(name) && !strcmp(gext(name),".gz")) )
-    gio = gopenstdio(name, mode);
-  else if( (*name == '-') && ((*(name+1) == '\0') || (*(name+1) == '[')) )
-    gio = gopenstdio(name, mode);
   /* look for a socket, in the form machine:port */
   else if( gioparseipport(name, &ip, &port) )
     gio = gopensocket(name, mode);
@@ -2021,7 +2030,7 @@ void gfreebuf(gio, buf)
      void *buf;
 #endif
 {
-  if( (gio==NULL) )
+  if( gio==NULL )
     return;
 
   switch(gio->type){
@@ -2115,8 +2124,10 @@ void gclose(gio)
       close(gio->ifd);
     if( gio->ofd >=0 )
       close(gio->ofd);
+#if HAVE_MINGW32==0
     if( gio->pid )
       waitpid(gio->pid, &status, 0);
+#endif
     break;
   case GIO_SOCKET:
     if( gio->ifd >=0 )
@@ -2209,7 +2220,7 @@ void gerror(va_alist) va_dcl
     vsnprintf(_gerrors, SZ_LINE-1, tbuf, args);
     /* if the error flag is positive, we output immediately */
     if( (fd != NULL) && _gerror ){
-      fprintf(fd, _gerrors);
+      fputs(_gerrors, fd);
       fflush(fd);
     }
     /* if the error flag is set high, we exit */
@@ -2281,7 +2292,7 @@ void gwarning(va_alist) va_dcl
     vsnprintf(_gwarnings, SZ_LINE-1, tbuf, args);
     /* if the warning flag is positive, we output immediately */
     if( (fd != NULL) && _gwarning ){
-      fprintf(fd, _gwarnings);
+      fputs(_gwarnings, fd);
       fflush(fd);
     }
 }
