@@ -1,8 +1,8 @@
 /*** File wcslib/imio.c
- *** January 8, 2007
- *** By Doug Mink, dmink@cfa.harvard.edu
+ *** October 30, 2012
+ *** By Jessica Mink, jmink@cfa.harvard.edu
  *** Harvard-Smithsonian Center for Astrophysics
- *** Copyright (C) 1996-2007
+ *** Copyright (C) 1996-2012
  *** Smithsonian Astrophysical Observatory, Cambridge, MA, USA
 
     This library is free software; you can redistribute it and/or
@@ -20,8 +20,8 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
     Correspondence concerning WCSTools should be addressed as follows:
-           Internet email: dmink@cfa.harvard.edu
-           Postal address: Doug Mink
+           Internet email: jmink@cfa.harvard.edu
+           Postal address: Jessica Mink
                            Smithsonian Astrophysical Observatory
                            60 Garden St.
                            Cambridge, MA 02138 USA
@@ -40,8 +40,10 @@
  *		Copy pixel into 2D image of any numeric type (0,0 lower left)
  * Subroutine:	addpix1 (image, bitpix, w, h, bz, bs, x, y, dpix)
  *		Add pixel into 2D image of any numeric type (1,1 lower left)
- * Subroutine:	maxvec (image, bitpix, bz, bs, pix1, npix, dmax)
+ * Subroutine:	maxvec (image, bitpix, bz, bs, pix1, npix)
  *		Get maximum of vector from 2D image of any numeric type
+ * Subroutine:	minvec (image, bitpix, bz, bs, pix1, npix)
+ *		Get minimum of vector from 2D image of any numeric type
  * Subroutine:	getvec (image, bitpix, bz, bs, pix1, npix, dvec)
  *		Get vector from 2D image of any numeric type
  * Subroutine:	putvec (image, bitpix, bz, bs, pix1, npix, dvec)
@@ -118,6 +120,7 @@ int	y;		/* Zero-based vertical pixel number */
 {
     short *im2;
     int *im4;
+    unsigned char *im1;
     unsigned short *imu;
     float *imr;
     double *imd;
@@ -133,7 +136,8 @@ int	y;		/* Zero-based vertical pixel number */
     switch (bitpix) {
 
 	case 8:
-	  dpix = (double) image[(y*w) + x];
+	  im1 = (unsigned char *)image;
+	  dpix = (double) im1[(y*w) + x];
 	  break;
 
 	case 16:
@@ -212,11 +216,12 @@ int	y;
 double	dpix;
 
 {
-    short *im2;
-    int *im4;
-    unsigned short *imu;
-    float *imr;
     double *imd;
+    float *imr;
+    int *im4;
+    short *im2;
+    unsigned short *imu;
+    unsigned char *im1;
 
 /* Return if coordinates are not inside image */
     if (x < 0 || x >= w)
@@ -230,10 +235,11 @@ double	dpix;
     switch (bitpix) {
 
 	case 8:
+	    im1 = (unsigned char *)image;
 	    if (dpix < 0)
-		image[(y*w) + x] = (char) (dpix - 0.5);
+		im1[(y*w) + x] = (unsigned char) (dpix - 0.5);
 	    else
-		image[(y*w) + x] = (char) (dpix + 0.5);
+		im1[(y*w) + x] = (unsigned char) (dpix + 0.5);
 	    break;
 
 	case 16:
@@ -316,11 +322,12 @@ int	y;		/* Zero-based vertical pixel number */
 double	dpix;		/* Value to add to pixel */
 
 {
-    short *im2;
-    int *im4;
-    unsigned short *imu;
-    float *imr;
     double *imd;
+    float *imr;
+    int *im4;
+    short *im2;
+    unsigned short *imu;
+    unsigned char *im1;
     int ipix;
 
 /* Return if coordinates are not inside image */
@@ -336,10 +343,11 @@ double	dpix;		/* Value to add to pixel */
     switch (bitpix) {
 
 	case 8:
+	    im1 = (unsigned char *)image;
 	    if (dpix < 0)
-		image[ipix] = image[ipix] + (char) (dpix - 0.5);
+		image[ipix] = im1[ipix] + (unsigned char) (dpix - 0.5);
 	    else
-		image[ipix] = image[ipix] + (char) (dpix + 0.5);
+		image[ipix] = im1[ipix] + (unsigned char) (dpix + 0.5);
 	    break;
 
 	case 16:
@@ -399,11 +407,12 @@ int	w2;		/* Number of horizontal pixels in output image */
 int	x2, y2;		/* Row and column for output pixel */
 
 {
-    short *ims1, *ims2;
-    int *imi1, *imi2;
-    unsigned short *imu1, *imu2;
-    float rpix, *imr1, *imr2;
     double dpix, *imd1, *imd2;
+    float rpix, *imr1, *imr2;
+    int *imi1, *imi2;
+    short *ims1, *ims2;
+    unsigned short *imu1, *imu2;
+    unsigned char *imc1, *imc2;
 
     if (x1 < 0 || x2 < 0 || x1 >= w1 || x2 >= w2)
 	return;
@@ -413,61 +422,59 @@ int	x2, y2;		/* Row and column for output pixel */
     switch (bitpix1) {
 
 	case 8:
+	    imc1 = (unsigned char *)image1;
 	    switch (bitpix2) {
 		case 8:
-		    image2[(y2*w2) + x2] = image1[(y1*w1) + x1];
+		    imc2 = (unsigned char *)image2;
+		    imc2[(y2*w2) + x2] = imc1[(y1*w1) + x1];
 		    break;
 		case 16:
 		    ims2 = (short *)image2;
-		    ims2[(y2*w2) + x2] = image1[(y1*w1) + x1];
+		    ims2[(y2*w2) + x2] = (short) imc1[(y1*w1) + x1];
 		    break;
 		case 32:
 		    imi2 = (int *)image2;
-		    imi2[(y2*w2) + x2] = (int) image1[(y1*w1) + x1];
+		    imi2[(y2*w2) + x2] = (int) imc1[(y1*w1) + x1];
 		    break;
 		case -16:
 		    imu2 = (unsigned short *)image2;
-		    imu2[(y2*w2) + x2] = (unsigned short) image1[(y1*w1) + x1];
+		    imu2[(y2*w2) + x2] = (unsigned short) imc1[(y1*w1) + x1];
 		    break;
 		case -32:
 		    imr2 = (float *)image2;
-		    imr2[(y2*w2) + x2] = (float) image1[(y1*w1) + x1];
+		    imr2[(y2*w2) + x2] = (float) imc1[(y1*w1) + x1];
 		    break;
 		case -64:
 		    imd2 = (double *)image2;
-		    imd2[(y2*w2) + x2] = (double) image1[(y1*w1) + x1];
+		    imd2[(y2*w2) + x2] = (double) imc1[(y1*w1) + x1];
 		    break;
 		}
 	    break;
 
 	case 16:
+	    ims1 = (short *)image1;
 	    switch (bitpix2) {
 		case 8:
-		    ims1 = (short *)image1;
-		    image2[(y2*w2) + x2] = (char) ims1[(y1*w1) + x1];
+		    imc2 = (unsigned char *)image1;
+		    imc2[(y2*w2) + x2] = (unsigned char) ims1[(y1*w1) + x1];
 		    break;
 		case 16:
-		    ims1 = (short *)image1;
 		    ims2 = (short *)image2;
 		    ims2[(y2*w2) + x2] = ims1[(y1*w1) + x1];
 		    break;
 		case 32:
-		    ims1 = (short *)image1;
 		    imi2 = (int *)image2;
 		    imi2[(y2*w2) + x2] = (int) ims1[(y1*w1) + x1];
 		    break;
 		case -16:
-		    ims1 = (short *)image1;
 		    imu2 = (unsigned short *)image2;
 		    imu2[(y2*w2) + x2] = (unsigned short) ims1[(y1*w1) + x1];
 		    break;
 		case -32:
-		    ims1 = (short *)image1;
 		    imr2 = (float *)image2;
 		    imr2[(y2*w2) + x2] = (float) ims1[(y1*w1) + x1];
 		    break;
 		case -64:
-		    ims1 = (short *)image1;
 		    imd2 = (double *)image2;
 		    imd2[(y2*w2) + x2] = (double) ims1[(y1*w1) + x1];
 		    break;
@@ -475,33 +482,29 @@ int	x2, y2;		/* Row and column for output pixel */
 	    break;
 
 	case 32:
+	    imi1 = (int *)image1;
 	    switch (bitpix2) {
 		case 8:
-		    imi1 = (int *)image1;
-		    image2[(y2*w2) + x2] = (char) imi1[(y1*w1) + x1];
+		    imc2 = (unsigned char *)image2;
+		    imc2[(y2*w2) + x2] = (unsigned char) imi1[(y1*w1) + x1];
 		    break;
 		case 16:
-		    imi1 = (int *)image1;
 		    ims2 = (short *)image2;
 		    ims2[(y2*w2) + x2] = (short) imi1[(y1*w1) + x1];
 		    break;
 		case 32:
-		    imi1 = (int *)image1;
 		    imi2 = (int *)image2;
 		    imi2[(y2*w2) + x2] = imi1[(y1*w1) + x1];
 		    break;
 		case -16:
-		    imi1 = (int *)image1;
 		    imu2 = (unsigned short *)image2;
 		    imu2[(y2*w2) + x2] = (unsigned short) imi1[(y1*w1) + x1];
 		    break;
 		case -32:
-		    imi1 = (int *)image1;
 		    imr2 = (float *)image2;
 		    imr2[(y2*w2) + x2] = (float) imi1[(y1*w1) + x1];
 		    break;
 		case -64:
-		    imi1 = (int *)image1;
 		    imd2 = (double *)image2;
 		    imd2[(y2*w2) + x2] = (double) imi1[(y1*w1) + x1];
 		    break;
@@ -509,33 +512,29 @@ int	x2, y2;		/* Row and column for output pixel */
 	    break;
 
 	case -16:
+	    imu1 = (unsigned short *)image1;
 	    switch (bitpix2) {
 		case 8:
-		    imu1 = (unsigned short *)image1;
-		    image2[(y2*w2) + x2] = (char) imu1[(y1*w1) + x1];
+		    imc2 = (unsigned char *)image2;
+		    imc2[(y2*w2) + x2] = (unsigned char) imu1[(y1*w1) + x1];
 		    break;
 		case 16:
-		    imu1 = (unsigned short *)image1;
 		    ims2 = (short *)image2;
 		    ims2[(y2*w2) + x2] = (short) imu1[(y1*w1) + x1];
 		    break;
 		case 32:
-		    imu1 = (unsigned short *)image1;
 		    imi2 = (int *)image2;
 		    imi2[(y2*w2) + x2] = (int) imu1[(y1*w1) + x1];
 		    break;
 		case -16:
-		    imu1 = (unsigned short *)image1;
 		    imu2 = (unsigned short *)image2;
 		    imu2[(y2*w2) + x2] = imu1[(y1*w1) + x1];
 		    break;
 		case -32:
-		    imu1 = (unsigned short *)image1;
 		    imr2 = (float *)image2;
 		    imr2[(y2*w2) + x2] = (float) imu1[(y1*w1) + x1];
 		    break;
 		case -64:
-		    imu1 = (unsigned short *)image1;
 		    imd2 = (double *)image2;
 		    imd2[(y2*w2) + x2] = (double) imu1[(y1*w1) + x1];
 		    break;
@@ -547,10 +546,11 @@ int	x2, y2;		/* Row and column for output pixel */
 	    rpix = imr1[(y1*w1) + x1];
 	    switch (bitpix2) {
 		case 8:
+		    imc2 = (unsigned char *)image2;
 		    if (rpix < 0.0)
-			image2[(y2*w2) + x2] = (char) (rpix - 0.5);
+			imc2[(y2*w2) + x2] = (unsigned char) 0;
 		    else
-			image2[(y2*w2) + x2] = (char) (rpix + 0.5);
+			imc2[(y2*w2) + x2] = (unsigned char) (rpix + 0.5);
 		    break;
 		case 16:
 		    ims2 = (short *)image2;
@@ -589,11 +589,11 @@ int	x2, y2;		/* Row and column for output pixel */
 	    dpix = imd1[(y1*w1) + x1];
 	    switch (bitpix2) {
 		case 8:
-		    imd1 = (double *)image1;
+		    imc2 = (unsigned char *)image2;
 		    if (dpix < 0.0)
-			image2[(y2*w2) + x2] = (char) (dpix - 0.5);
+			imc2[(y2*w2) + x2] = (unsigned char) 0;
 		    else
-			image2[(y2*w2) + x2] = (char) (dpix + 0.5);
+			imc2[(y2*w2) + x2] = (unsigned char) (dpix + 0.5);
 		    break;
 		case 16:
 		    ims2 = (short *)image2;
@@ -646,75 +646,82 @@ int	pix1;		/* Offset of first pixel to check */
 int	npix;		/* Number of pixels to check */
 
 {
-    short *im2;
-    int *im4;
-    unsigned short *imu;
-    float *imr;
+    short *im2, imax2, ip2;
+    int *im4, imax4, ip4;
+    unsigned short *imu, imaxu, ipu;
+    float *imr, imaxr, ipr;
     double *imd;
     double dmax = 0.0;
-    double dpix;
+    double ipd;
     int ipix, pix2;
+    unsigned char *imc, imaxc, ipc;
 
     pix2 = pix1 + npix;
 
     switch (bitpix) {
 
 	case 8:
-	    dmax = (double) *(image + pix1);
+	    imc = (unsigned char *)(image);
+	    imaxc = *(imc + pix1);
 	    for (ipix = pix1; ipix < pix2; ipix++) {
-		dpix = (double) *(image + ipix);
-		if (dpix > dmax)
-		    dmax = dpix;
+		ipc = *(imc + ipix);
+		if (ipc > imaxc)
+		    imaxc = ipc;
 		}
+	    dmax = (double) imaxc;
 	    break;
 
 	case 16:
 	    im2 = (short *)image;
-	    dmax = (double) *(im2 + pix1);
+	    imax2 = *(im2 + pix1);
 	    for (ipix = pix1; ipix < pix2; ipix++) {
-		dpix = (double) *(im2 + ipix);
-		if (dpix > dmax)
-		    dmax = dpix;
+		ip2 = *(im2 + ipix);
+		if (ip2 > imax2)
+		    imax2 = ip2;
 		}
+	    dmax = (double) imax2;
 	    break;
 
 	case 32:
 	    im4 = (int *)image;
-	    dmax = (double) *(im4 + pix1);
+	    imax4 = *(im4 + pix1);
 	    for (ipix = pix1; ipix < pix2; ipix++) {
-		dpix = (double) *(im4 + ipix);
-		if (dpix > dmax)
-		    dmax = dpix;
+		ip4 = *(im4 + ipix);
+		if (ip4 > imax4)
+		    imax4 = ip4;
 		}
+	    dmax = (double) imax4;
 	    break;
 
 	case -16:
 	    imu = (unsigned short *)image;
-	    dmax = (double) *(imu + pix1);
+	    imaxu = *(imu + pix1);
 	    for (ipix = pix1; ipix < pix2; ipix++) {
-		dpix = (double) *(imu + ipix);
-		if (dpix > dmax)
-		    dmax = dpix;
+		ipu = *(imu + ipix);
+		if (ipu > imaxu)
+		    imaxu = ipu;
 		}
+	    dmax = (double) imaxu;
 	    break;
 
 	case -32:
 	    imr = (float *)image;
-	    dmax = (double) *(imr + pix1);
+	    imaxr = *(imr + pix1);
 	    for (ipix = pix1; ipix < pix2; ipix++) {
-		dpix = (double) *(imr + ipix);
-		if (dpix > dmax)
-		    dmax = dpix;
+		ipr = *(imr + ipix);
+		if (ipr > imaxr)
+		    imax2 = ipr;
 		}
+	    dmax = (double) imaxr;
 	    break;
 
 	case -64:
 	    imd = (double *)image;
 	    dmax = *(imd + pix1);
 	    for (ipix = pix1; ipix < pix2; ipix++) {
-		dpix = *(imd + ipix);
-		if (dpix > dmax)
-		    dmax = dpix;
+		ipd = *(imd + ipix);
+		if (ipd > dmax)
+		    dmax = ipd;
 		}
 	    break;
 
@@ -728,6 +735,109 @@ int	npix;		/* Number of pixels to check */
 }
 
 
+/* MINVEC -- Get minimum value in vector from 2D image of any numeric type */
+
+double
+minvec (image, bitpix, bzero, bscale, pix1, npix)
+
+char	*image;		/* Image array from which to read vector */
+int	bitpix;		/* Number of bits per pixel in image */
+			/*  16 = short, -16 = unsigned short, 32 = int */
+			/* -32 = float, -64 = double */
+double  bzero;		/* Zero point for pixel scaling */
+double  bscale;		/* Scale factor for pixel scaling */
+int	pix1;		/* Offset of first pixel to check */
+int	npix;		/* Number of pixels to check */
+
+{
+    short *im2, imin2, ip2;
+    int *im4, imin4, ip4;
+    unsigned short *imu, iminu, ipu;
+    float *imr, iminr, ipr;
+    double *imd, ipd;
+    double dmin = 0.0;
+    int ipix, pix2;
+    unsigned char *imc, cmin, cp;
+
+    pix2 = pix1 + npix;
+
+    switch (bitpix) {
+
+	case 8:
+	    imc = (unsigned char *)image;
+	    cmin = *(imc + pix1);
+	    for (ipix = pix1; ipix < pix2; ipix++) {
+		cp = *(imc + ipix);
+		if (cp < cmin)
+		    cmin = cp;
+		}
+	    dmin = (double) cmin;
+	    break;
+
+	case 16:
+	    im2 = (short *)image + pix1;
+	    imin2 = *im2;
+	    for (ipix = pix1; ipix < pix2; ipix++) {
+		ip2 = *(im2 + ipix);
+		if (ip2 < imin2)
+		    imin2 = ip2;
+		}
+	    dmin = (double) imin2;
+	    break;
+
+	case 32:
+	    im4 = (int *)image;
+	    imin4 = *(im4 + pix1);
+	    for (ipix = pix1; ipix < pix2; ipix++) {
+		ip4 = *(im4 + ipix);
+		if (ip4 < imin4)
+		    imin4 = ip4;
+		}
+	    dmin = (double) imin4;
+	    break;
+
+	case -16:
+	    imu = (unsigned short *)image;
+	    iminu = *(imu + pix1);
+	    for (ipix = pix1; ipix < pix2; ipix++) {
+		ipu = *(imu + ipix);
+		if (ipu < iminu)
+		    iminu = ipu;
+		}
+	    dmin = (double) iminu;
+	    break;
+
+	case -32:
+	    imr = (float *)image;
+	    iminr = *(imr + pix1);
+	    for (ipix = pix1; ipix < pix2; ipix++) {
+		ipr = *(imr + ipix);
+		if (ipr < iminr)
+		    iminr = ipr;
+		}
+	    dmin = (double) iminr;
+	    break;
+
+	case -64:
+	    imd = (double *)image;
+	    dmin = *(imd + pix1);
+	    for (ipix = pix1; ipix < pix2; ipix++) {
+		ipd = *(imd + ipix);
+		if (ipd < dmin)
+		    dmin = ipd;
+		}
+	    break;
+
+	}
+
+    /* Scale data if either BZERO or BSCALE keyword has been set */
+    if (scale && (bzero != 0.0 || bscale != 1.0))
+	dmin = (dmin * bscale) + bzero;
+
+    return (dmin);
+}
+
+
 /* ADDVEC -- Add constant to pixel values in 2D image of any numeric type */
 
 void
@@ -744,7 +854,7 @@ int	npix;		/* Number of pixels to extract */
 double	dpix;		/* Value to add to pixels */
 
 {
-    char *imc, ccon;
+    unsigned char *imc, ccon;
     short *im2, jcon;
     int *im4, icon;
     unsigned short *imu, ucon;
@@ -760,11 +870,11 @@ double	dpix;		/* Value to add to pixels */
     switch (bitpix) {
 
 	case 8:
-	    imc = image + pix1;
+	    imc = (unsigned char *) (image + pix1);
 	    if (dpix < 0)
-		ccon = (char) (dpix - 0.5);
+		ccon = (unsigned char) (dpix - 0.5);
 	    else
-		ccon = (char) (dpix + 0.5);
+		ccon = (unsigned char) (dpix + 0.5);
 	    for (ipix = pix1; ipix < pix2; ipix++)
 		*imc++ += ccon;
 	    break;
@@ -1425,4 +1535,9 @@ imswapped ()
  * Jun 20 2006	Fix typos masquerading as unitialized variables
  *
  * Jan  8 2007	Include fitsfile.h instead of imio.h
+ * Jun 11 2007	Add minvec() and speed up maxvec()
+ *
+ * Apr 12 2012	Fix 8-bit variables to be unsigned char
+ * Oct 19 2012	Fix errors with character images in minvec() and maxvec()
+ * Oct 30 2012	Fix errors with short images in minvec() and maxvec()
  */
