@@ -1,8 +1,8 @@
 /*** File libwcs/wcsinit.c
- *** January 8, 2007
- *** By Doug Mink, dmink@cfa.harvard.edu
+ *** October 19, 2012
+ *** By Jessica Mink, jmink@cfa.harvard.edu
  *** Harvard-Smithsonian Center for Astrophysics
- *** Copyright (C) 1998-2007
+ *** Copyright (C) 1998-2012
  *** Smithsonian Astrophysical Observatory, Cambridge, MA, USA
 
     This library is free software; you can redistribute it and/or
@@ -20,8 +20,8 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
     Correspondence concerning WCSTools should be addressed as follows:
-           Internet email: dmink@cfa.harvard.edu
-           Postal address: Doug Mink
+           Internet email: jmink@cfa.harvard.edu
+           Postal address: Jessica Mink
                            Smithsonian Astrophysical Observatory
                            60 Garden St.
                            Cambridge, MA 02138 USA
@@ -235,22 +235,25 @@ char *wchar;		/* Suffix character for one of multiple WCS */
     char temp[80];
     char wcsname[64];	/* Name of WCS depended on by current WCS */
     char mchar;
+    char cspace = (char) ' ';
+    char cnull = (char) 0;
     double mjd;
     double rot;
     double ut;
     int nax;
     int twod;
-    int iszpx = 0;
     extern int tnxinit();
+    extern int zpxinit();
     extern int platepos();
     extern int dsspos();
+    void invert_wcs();
 
     wcs = (struct WorldCoor *) calloc (1, sizeof(struct WorldCoor));
 
     /* Set WCS character and name in structure */
     mchar = wchar[0];
     if (mchar == ' ')
-	mchar = (char) 0;
+	mchar = cnull;
     wcs->wcschar = mchar;
     if (hgetsc (hstring, "WCSNAME", &mchar, 63, wcsname)) {
 	wcs->wcsname = (char *) calloc (strlen (wcsname)+2, 1);
@@ -300,22 +303,53 @@ char *wchar;		/* Suffix character for one of multiple WCS */
     wcs->lin.naxis = naxes;
     wcs->nxpix = 0;
     hgetr8 (hstring, "NAXIS1", &wcs->nxpix);
+    if (wcs->nxpix < 1)
+	hgetr8 (hstring, "IMAGEW", &wcs->nxpix);
     if (wcs->nxpix < 1) {
-	setwcserr ("WCSINIT: No NAXIS1 keyword");
+	setwcserr ("WCSINIT: No NAXIS1 or IMAGEW keyword");
 	wcsfree (wcs);
 	return (NULL);
 	}
     wcs->nypix = 0;
     hgetr8 (hstring, "NAXIS2", &wcs->nypix);
+    if (wcs->nypix < 1)
+	hgetr8 (hstring, "IMAGEH", &wcs->nypix);
+    if (naxes > 1 && wcs->nypix < 1) {
+	setwcserr ("WCSINIT: No NAXIS2 or IMAGEH keyword");
+	wcsfree (wcs);
+	return (NULL);
+	}
 
     /* Reset number of axes to only those with dimension greater than one */
     nax = 0;
     for (i = 0; i < naxes; i++) {
+
+	/* Check for number of pixels in axis more than one */
 	strcpy (keyword, "NAXIS");
 	sprintf (temp, "%d", i+1);
 	strcat (keyword, temp);
-	if (!hgeti4 (hstring, keyword, &j))
-	    fprintf (stderr,"WCSINIT: Missing keyword %s assumed 1\n",keyword);
+	if (!hgeti4 (hstring, keyword, &j)) {
+	    if (i == 0 && wcs->nxpix > 1) {
+		/* fprintf (stderr,"WCSINIT: Missing keyword %s set to %.0f from IMAGEW\n",
+			 keyword, wcs->nxpix); */
+		j = wcs->nxpix;
+		}
+	    else if (i == 1 && wcs->nypix > 1) {
+		/* fprintf (stderr,"WCSINIT: Missing keyword %s set to %.0f from IMAGEH\n",
+			 keyword, wcs->nypix); */
+		j = wcs->nypix;
+		}
+	    else
+		fprintf (stderr,"WCSINIT: Missing keyword %s assumed 1\n",keyword);
+	    }
+
+	/* Check for TAB WCS in axis */
+	strcpy (keyword, "CTYPE");
+	strcat (keyword, temp);
+	if (hgets (hstring, keyword, 16, temp)) {
+	    if (strsrch (temp, "-TAB"))
+		j = 0;
+	    }
 	if (j > 1) nax = nax + 1;
 	}
     naxes = nax;
@@ -325,6 +359,8 @@ char *wchar;		/* Suffix character for one of multiple WCS */
     hgets (hstring, "INSTRUME", 16, wcs->instrument);
     hgeti4 (hstring, "DETECTOR", &wcs->detector);
     wcs->wcsproj = getdefwcs();
+    wcs->logwcs = 0;
+    hgeti4 (hstring, "DC-FLAG", &wcs->logwcs);
 
     /* Initialize rotation matrices */
     for (i = 0; i < 81; i++) wcs->pc[i] = 0.0;
@@ -364,30 +400,13 @@ char *wchar;		/* Suffix character for one of multiple WCS */
 
     /* World coordinate system reference coordinate information */
     if (hgetsc (hstring, "CTYPE1", &mchar, 16, ctype1)) {
-	if (!strncmp (ctype1+5,"ZPX", 3)) {
-	    iszpx = 1;
-	    ctype1[7] = 'N';
-
-	    /* IRAF ZPX parameters for ZPN projection */
-	    for (i = 0; i < 10; i++) {
-		sprintf (keyword,"projp%d",i);
-		mgetr8 (hstring, "WAT1",keyword, &wcs->prj.p[i]);
-		}
-	    }
-	else
-	    iszpx = 0;
 
 	/* Read second coordinate type */
 	strcpy (ctype2, ctype1);
 	if (!hgetsc (hstring, "CTYPE2", &mchar, 16, ctype2))
 	    twod = 0;
-	else {
+	else
 	    twod = 1;
-	    if (!strncmp (ctype2+5,"ZPX", 3)) {
-		iszpx = 1;
-		ctype2[7] = 'N';
-		}
-	    }
 	strcpy (wcs->ctype[0], ctype1);
 	strcpy (wcs->ctype[1], ctype2);
 	if (strsrch (ctype2, "LAT") || strsrch (ctype2, "DEC"))
@@ -518,21 +537,37 @@ char *wchar;		/* Suffix character for one of multiple WCS */
 		}
 	    }
 
-	/* If ZPX, read coefficients from WATi keyword */
-	if (iszpx) {
-	    char mkey[8];
-	    sprintf (mkey,"WAT%d", ilat);
-	    for (i = 0; i < 10; i++) {
-		wcs->prj.p[i] = 0.0;
-		sprintf (keyword,"projp%d",i);
-		mgetr8 (hstring, mkey, keyword, &wcs->prj.p[i]);
+	/* Initialize TNX, defaulting to TAN if there is a problem */
+	if (wcs->prjcode == WCS_TNX) {
+	    if (tnxinit (hstring, wcs)) {
+		wcs->ctype[0][6] = 'A';
+		wcs->ctype[0][7] = 'N';
+		wcs->ctype[1][6] = 'A';
+		wcs->ctype[1][7] = 'N';
+		wcs->prjcode = WCS_TAN;
 		}
 	    }
 
+	/* Initialize ZPX, defaulting to ZPN if there is a problem */
+	if (wcs->prjcode == WCS_ZPX) {
+	    if (zpxinit (hstring, wcs)) {
+		wcs->ctype[0][7] = 'N';
+		wcs->ctype[1][7] = 'N';
+		wcs->prjcode = WCS_ZPN;
+		}
+	    }
+
+	/* Set TPV to TAN as SCAMP coefficients will be added below */
+	if (wcs->prjcode == WCS_TPV) {
+	    wcs->ctype[0][6] = 'A';
+	    wcs->ctype[0][7] = 'N';
+	    wcs->ctype[1][6] = 'A';
+	    wcs->ctype[1][7] = 'N';
+	    wcs->prjcode = WCS_TAN;
+	    }
 
 	/* Coordinate reference frame, equinox, and epoch */
-	if (strncmp (wcs->ptype,"LINEAR",6) &&
-	    strncmp (wcs->ptype,"PIXEL",5))
+	if (wcs->wcsproj > 0)
 	    wcseqm (hstring, wcs, &mchar);
 	wcsioset (wcs);
 
@@ -668,7 +703,7 @@ char *wchar;		/* Suffix character for one of multiple WCS */
 		pc[(i*naxes)+i] = 1.0;
 
 	    /* Read FITS WCS interim rotation matrix */
-	    if (!&mchar && hgetr8 (hstring,"PC001001",&pc[0]) != 0) {
+	    if (!mchar && hgetr8 (hstring,"PC001001",&pc[0]) != 0) {
 		k = 0;
 		for (i = 0; i < naxes; i++) {
 		    for (j = 0; j < naxes; j++) {
@@ -721,14 +756,49 @@ char *wchar;		/* Suffix character for one of multiple WCS */
 	    setwcserr ("WCSINIT: setting CDELT to 1");
 	    }
 
-	/* Initialize TNX, defaulting to TAN if there is a problem */
-	if (wcs->prjcode == WCS_TNX) {
-	    if (tnxinit (hstring, wcs)) {
-		wcs->ctype[0][6] = 'A';
-		wcs->ctype[0][7] = 'N';
-		wcs->ctype[1][6] = 'A';
-		wcs->ctype[1][7] = 'N';
-		wcs->prjcode = WCS_TAN;
+	/* SCAMP convention */
+	if (wcs->prjcode == WCS_TAN && wcs->naxis == 2) { 
+	    int n = 0;
+	    if (wcs->inv_x) {
+		poly_end(wcs->inv_x);
+		wcs->inv_x = NULL;
+		}
+	    if (wcs->inv_y) {
+		poly_end(wcs->inv_y);
+		wcs->inv_y = NULL;
+		}
+	    wcs->pvfail = 0;
+	    for (i = 0; i < (2*MAXPV); i++) {
+		wcs->projppv[i] = 0.0;
+		wcs->prj.ppv[i] = 0.0;
+		}
+	    for (k = 0; k < 2; k++) {
+		for (j = 0; j < MAXPV; j++) {
+		    sprintf(keyword, "PV%d_%d", k+1, j);
+		    if (hgetr8c(hstring, keyword,&mchar, &wcs->projppv[j+k*MAXPV]) == 0) {
+			wcs->projppv[j+k*MAXPV] = 0.0;
+			}
+		    else
+			n++;
+		    }
+		}
+
+	    /* If any PVi_j are set, add them to the structure */
+	    if (n > 0) {
+		n = 0;
+
+		for (k = MAXPV; k >= 0; k--) {
+		    /* lat comes first for compatibility reasons */
+		    wcs->prj.ppv[k] = wcs->projppv[k+wcs->wcsl.lat*MAXPV];
+		    wcs->prj.ppv[k+MAXPV] = wcs->projppv[k+wcs->wcsl.lng*MAXPV];
+		    if (!n && (wcs->prj.ppv[k] || wcs->prj.ppv[k+MAXPV]))  {
+			n = k+1;
+			}
+		    }
+		invert_wcs(wcs);
+
+		/* Need to call tanset again */
+		wcs->cel.flag = 0;
 		}
 	    }
 
@@ -763,7 +833,7 @@ char *wchar;		/* Suffix character for one of multiple WCS */
 	wcs->wcson = 1;
 	}
 
-    else if (&mchar != (char) 0) {
+    else if (mchar != cnull && mchar != cspace) {
 	(void) sprintf (temp, "WCSINITC: No image scale for WCS %c", mchar);
 	setwcserr (temp);
 	wcsfree (wcs);
@@ -938,7 +1008,7 @@ char *wchar;		/* Suffix character for one of multiple WCS */
 		*ic = ' ';
 	    ic = strchr (temp, ']');
 	    if (ic != NULL)
-		*ic = (char) 0;
+		*ic = cnull;
 	    sscanf (temp, "%d %d %d %d", &idx1, &idx2, &idy1, &idy2);
 	    dxrefpix = 0.5 * (double) (idx1 + idx2 - 1);
 	    dyrefpix = 0.5 * (double) (idy1 + idy2 - 1);
@@ -954,7 +1024,7 @@ char *wchar;		/* Suffix character for one of multiple WCS */
 		*ic = ' ';
 	    ic = strchr (temp, ']');
 	    if (ic != NULL)
-		*ic = (char) 0;
+		*ic = cnull;
 	    sscanf (temp, "%d %d %d %d", &ix1, &ix2, &iy1, &iy2);
 	    wcs->crpix[0] = dxrefpix - (double) (ix1 - 1);
 	    wcs->crpix[1] = dyrefpix - (double) (iy1 - 1);
@@ -1038,6 +1108,191 @@ char *wchar;		/* Suffix character for one of multiple WCS */
 }
 
 
+/******* invert_wcs ***********************************************************
+PROTO	void invert_wcs(wcsstruct *wcs)
+PURPOSE	Invert WCS projection mapping (using a polynomial).
+INPUT	WCS structure.
+OUTPUT	-.
+NOTES	.
+AUTHOR	E. Bertin (IAP)
+VERSION	06/11/2003
+ ***/
+
+void
+invert_wcs( struct WorldCoor *wcs)
+
+{
+    polystruct	*poly;
+    double pixin[NAXISPV],raw[NAXISPV],rawmin[NAXISPV];
+    double *outpos,*outpost, *lngpos,*lngpost;
+    double *latpos,*latpost,lngstep,latstep, rawsize, epsilon;
+    int	 group[] = {1,1};
+    /* Don't ask, this is needed by poly_init()! */
+    int	 i,j,lng,lat,deg,  maxflag;
+    char errstr[80];
+    double xmin;
+    double ymin;
+    double xmax;
+    double ymax;
+    double lngmin;
+    double latmin;
+
+    /* Check first that inversion is not straightforward */
+    lng = wcs->wcsl.lng;
+    lat = wcs->wcsl.lat;
+
+    if (wcs->naxis != NAXISPV) {
+	return;
+	}
+
+    if (strcmp(wcs->wcsl.pcode, "TAN") != 0) {
+	return;
+	}
+
+    if ((wcs->projppv[1+lng*MAXPV] == 0) &&
+	(wcs->projppv[1+lat*MAXPV] == 0)) {
+	return;
+	}
+
+    if (wcs->wcs != NULL) {
+	pix2wcs(wcs->wcs,0,0,&xmin,&ymin);
+	pix2wcs(wcs->wcs,wcs->nxpix,wcs->nypix,&xmax,&ymax);
+	}
+    else {
+	xmin = 0;
+	ymin = 0;
+	xmax = wcs->nxpix;
+	ymax = wcs->nypix;
+	}
+
+    /* We define x as "longitude" and y as "latitude" projections */
+    /* We assume that PCxx cross-terms with additional dimensions are small */
+    /* Sample the whole image with a regular grid */
+    if (lng == 0) {
+	lngstep = (xmax-xmin)/(WCS_NGRIDPOINTS-1.0);
+	lngmin  = xmin;
+	latstep = (ymax-ymin)/(WCS_NGRIDPOINTS-1.0);
+	latmin  = ymin;
+	}
+    else {
+	lngstep = (ymax-ymin)/(WCS_NGRIDPOINTS-1.0);
+	lngmin = ymin;
+	latstep = (xmax-xmin)/(WCS_NGRIDPOINTS-1.0);
+	latmin = xmin;
+	}
+
+    outpos = (double *)calloc(2*WCS_NGRIDPOINTS2,sizeof(double));
+    lngpos = (double *)calloc(WCS_NGRIDPOINTS2,sizeof(double));
+    latpos = (double *)calloc(WCS_NGRIDPOINTS2,sizeof(double));
+    raw[lat] = rawmin[lat] = 0.5+latmin;
+    raw[lng] = rawmin[lng] = 0.5+lngmin;
+    outpost = outpos;
+    lngpost = lngpos;
+    latpost = latpos;
+    for (j=WCS_NGRIDPOINTS; j--; raw[lat]+=latstep) {
+  	raw[lng] = rawmin[lng];
+	for (i=WCS_NGRIDPOINTS; i--; raw[lng]+=lngstep) {
+	    if (linrev(raw, &wcs->lin, pixin)) {
+		sprintf (errstr,"*Error*: incorrect linear conversion in %s",
+			 wcs->wcsl.pcode);
+		setwcserr (errstr);
+		}
+	    *(lngpost++) = pixin[lng];
+	    *(latpost++) = pixin[lat];
+	    raw_to_pv (&wcs->prj,pixin[lng],pixin[lat], outpost, outpost+1);
+	    outpost += 2;
+	    }
+	}
+
+    /* Invert "longitude" */
+    /* Compute the extent of the pixel in reduced projected coordinates */
+    linrev(rawmin, &wcs->lin, pixin);
+    pixin[lng] += S2D;
+    linfwd(pixin, &wcs->lin, raw);
+    rawsize = sqrt((raw[lng]-rawmin[lng])*(raw[lng]-rawmin[lng])
+                 +(raw[lat]-rawmin[lat])*(raw[lat]-rawmin[lat]))*D2S;
+    if (!rawsize) {
+	sprintf (errstr,"*Error*: incorrect linear conversion in %s",
+		 wcs->wcsl.pcode);
+	setwcserr (errstr);
+	}
+    epsilon = WCS_INVACCURACY/rawsize;
+
+    /* Find the lowest degree polynom */
+    poly = NULL;  /* to avoid gcc -Wall warnings */
+    maxflag = 1;
+    for (deg=1; deg<=WCS_INVMAXDEG && maxflag; deg++) {
+	if (deg>1) {
+	    poly_end(poly);
+	    }
+	poly = poly_init(group, 2, &deg, 1);
+	poly_fit(poly, outpos, lngpos, NULL, WCS_NGRIDPOINTS2, NULL);
+	maxflag = 0;
+	outpost = outpos;
+	lngpost = lngpos;
+  	for (i=WCS_NGRIDPOINTS2; i--; outpost+=2) {
+	    if (fabs(poly_func(poly, outpost)-*(lngpost++))>epsilon) {
+		maxflag = 1;
+		break;
+		}
+	    }
+	}
+    if (maxflag) {
+	setwcserr ("WARNING: Significant inaccuracy likely to occur in projection");
+	wcs->pvfail = 1;
+	}
+
+    /* Now link the created structure */
+    wcs->prj.inv_x = wcs->inv_x = poly;
+
+    /* Invert "latitude" */
+    /* Compute the extent of the pixel in reduced projected coordinates */
+    linrev(rawmin, &wcs->lin, pixin);
+    pixin[lat] += S2D;
+    linfwd(pixin, &wcs->lin, raw);
+    rawsize = sqrt((raw[lng]-rawmin[lng])*(raw[lng]-rawmin[lng])
+                 +(raw[lat]-rawmin[lat])*(raw[lat]-rawmin[lat]))*D2S;
+    if (!rawsize) {
+	sprintf (errstr,"*Error*: incorrect linear conversion in %s",
+		 wcs->wcsl.pcode);
+	setwcserr (errstr);
+	}
+    epsilon = WCS_INVACCURACY/rawsize;
+
+    /* Find the lowest degree polynom */
+    maxflag = 1;
+    for (deg=1; deg<=WCS_INVMAXDEG && maxflag; deg++) {
+	if (deg>1)
+	    poly_end(poly);
+	poly = poly_init(group, 2, &deg, 1);
+	poly_fit(poly, outpos, latpos, NULL, WCS_NGRIDPOINTS2, NULL);
+	maxflag = 0;
+	outpost = outpos;
+	latpost = latpos;
+	for (i=WCS_NGRIDPOINTS2; i--; outpost+=2) {
+	    if (fabs(poly_func(poly, outpost)-*(latpost++))>epsilon) {
+		maxflag = 1;
+		break;
+		}
+	    }
+	}
+    if (maxflag) {
+	setwcserr ("WARNING: Significant inaccuracy likely to occur in projection");
+	wcs->pvfail = 1;
+	}
+
+    /* Now link the created structure */
+    wcs->prj.inv_y = wcs->inv_y = poly;
+
+    /* Free memory */
+    free(outpos);
+    free(lngpos);
+    free(latpos);
+
+    return;
+}
+
+
 /* Set coordinate system of image, input, and output */
 
 static void
@@ -1336,4 +1591,21 @@ char	*mchar;		/* Suffix character for one of multiple WCS */
  *
  * Jan  4 2007	Drop declarations of wcsinitc() and wcsinitn() already in wcs.h
  * Jan  8 2007	Change WCS letter from char to char*
+ * Feb  1 2007	Read IRAF log wavelength flag DC-FLAG to wcs.logwcs
+ * Feb 15 2007	Check for wcs->wcsproj > 0 instead of CTYPEi != LINEAR or PIXEL
+ * Mar 13 2007	Try for RA, DEC, SECPIX if WCS character is space or null
+ * Apr 27 2007	Ignore axes with TAB WCS for now
+ * Oct 17 2007	Fix bug testing &mchar instead of mchar in if statement
+ *
+ * May  9 2008	Initialize TNX projection when projection types first set
+ * Jun 27 2008	If NAXIS1 and NAXIS2 not present, check for IMAGEW and IMAGEH
+ *
+ * Mar 24 2009	Fix dimension bug if NAXISi not present (fix from John Burns)
+ *
+ * Mar 11 2011	Add NOAO ZPX projection (Frank Valdes)
+ * Mar 18 2011	Add invert_wcs() by Emmanuel Bertin for SCAMP
+ * Mar 18 2011	Change Bertin's ARCSEC/DEG to S2D and DEG/ARCSEC to D2S
+ * Sep  1 2011	Add TPV as TAN with SCAMP PVs
+ *
+ * Oct 19 2012	Drop unused variable iszpx; fix bug in latmin assignment
  */
